# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_REPLACE_STRUCTURE_METADATA = Metadata(
    id="06b2d63161345b1accca9efe553084791baedc0f.workbench",
    name="cifti-replace-structure",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiReplaceStructureVolumeAllParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureVolumeAllParamsDictNoTag', {
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeAllParamsDictTagged = typing.TypedDict('CiftiReplaceStructureVolumeAllParamsDictTagged', {
    "@type": typing.Literal["volume-all"],
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeAllParamsDict = _CiftiReplaceStructureVolumeAllParamsDictNoTag | CiftiReplaceStructureVolumeAllParamsDictTagged


_CiftiReplaceStructureLabelParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureLabelParamsDictNoTag', {
    "structure": str,
    "label": InputPathType,
})
CiftiReplaceStructureLabelParamsDictTagged = typing.TypedDict('CiftiReplaceStructureLabelParamsDictTagged', {
    "@type": typing.Literal["label"],
    "structure": str,
    "label": InputPathType,
})
CiftiReplaceStructureLabelParamsDict = _CiftiReplaceStructureLabelParamsDictNoTag | CiftiReplaceStructureLabelParamsDictTagged


_CiftiReplaceStructureMetricParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureMetricParamsDictNoTag', {
    "structure": str,
    "metric": InputPathType,
})
CiftiReplaceStructureMetricParamsDictTagged = typing.TypedDict('CiftiReplaceStructureMetricParamsDictTagged', {
    "@type": typing.Literal["metric"],
    "structure": str,
    "metric": InputPathType,
})
CiftiReplaceStructureMetricParamsDict = _CiftiReplaceStructureMetricParamsDictNoTag | CiftiReplaceStructureMetricParamsDictTagged


_CiftiReplaceStructureVolumeParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureVolumeParamsDictNoTag', {
    "structure": str,
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeParamsDictTagged = typing.TypedDict('CiftiReplaceStructureVolumeParamsDictTagged', {
    "@type": typing.Literal["volume"],
    "structure": str,
    "volume": InputPathType,
    "from-cropped": bool,
})
CiftiReplaceStructureVolumeParamsDict = _CiftiReplaceStructureVolumeParamsDictNoTag | CiftiReplaceStructureVolumeParamsDictTagged


_CiftiReplaceStructureParamsDictNoTag = typing.TypedDict('_CiftiReplaceStructureParamsDictNoTag', {
    "cifti": str,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiReplaceStructureVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiReplaceStructureLabelParamsDict] | None],
    "metric": typing.NotRequired[list[CiftiReplaceStructureMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiReplaceStructureVolumeParamsDict] | None],
    "action": typing.NotRequired[str | None],
    "discard-unused-labels": bool,
})
CiftiReplaceStructureParamsDictTagged = typing.TypedDict('CiftiReplaceStructureParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-replace-structure"],
    "cifti": str,
    "direction": str,
    "volume-all": typing.NotRequired[CiftiReplaceStructureVolumeAllParamsDict | None],
    "label": typing.NotRequired[list[CiftiReplaceStructureLabelParamsDict] | None],
    "metric": typing.NotRequired[list[CiftiReplaceStructureMetricParamsDict] | None],
    "volume": typing.NotRequired[list[CiftiReplaceStructureVolumeParamsDict] | None],
    "action": typing.NotRequired[str | None],
    "discard-unused-labels": bool,
})
CiftiReplaceStructureParamsDict = _CiftiReplaceStructureParamsDictNoTag | CiftiReplaceStructureParamsDictTagged


def cifti_replace_structure_volume_all(
    volume: InputPathType,
    from_cropped: bool = False,
) -> CiftiReplaceStructureVolumeAllParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume: the input volume.
        from_cropped: the input is cropped to the size of the data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume-all",
        "volume": volume,
        "from-cropped": from_cropped,
    }
    return params


def cifti_replace_structure_volume_all_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureVolumeAllParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("from-cropped", False) is None:
        raise StyxValidationError("`from-cropped` must not be None")
    if not isinstance(params["from-cropped"], bool):
        raise StyxValidationError(f'`from-cropped` has the wrong type: Received `{type(params.get("from-cropped", False))}` expected `bool`')


def cifti_replace_structure_volume_all_cargs(
    params: CiftiReplaceStructureVolumeAllParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume-all",
        execution.input_file(params.get("volume", None))
    ])
    if params.get("from-cropped", False):
        cargs.append("-from-cropped")
    return cargs


def cifti_replace_structure_label(
    structure: str,
    label: InputPathType,
) -> CiftiReplaceStructureLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        label: the input label file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "structure": structure,
        "label": label,
    }
    return params


def cifti_replace_structure_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')


def cifti_replace_structure_label_cargs(
    params: CiftiReplaceStructureLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        params.get("structure", None),
        execution.input_file(params.get("label", None))
    ])
    return cargs


def cifti_replace_structure_metric(
    structure: str,
    metric: InputPathType,
) -> CiftiReplaceStructureMetricParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        metric: the input metric.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "metric",
        "structure": structure,
        "metric": metric,
    }
    return params


def cifti_replace_structure_metric_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureMetricParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')


def cifti_replace_structure_metric_cargs(
    params: CiftiReplaceStructureMetricParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-metric",
        params.get("structure", None),
        execution.input_file(params.get("metric", None))
    ])
    return cargs


def cifti_replace_structure_volume(
    structure: str,
    volume: InputPathType,
    from_cropped: bool = False,
) -> CiftiReplaceStructureVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        structure: the structure to replace the data of.
        volume: the input volume.
        from_cropped: the input is cropped to the size of the component.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "volume",
        "structure": structure,
        "volume": volume,
        "from-cropped": from_cropped,
    }
    return params


def cifti_replace_structure_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("structure", None) is None:
        raise StyxValidationError("`structure` must not be None")
    if not isinstance(params["structure"], str):
        raise StyxValidationError(f'`structure` has the wrong type: Received `{type(params.get("structure", None))}` expected `str`')
    if params.get("volume", None) is None:
        raise StyxValidationError("`volume` must not be None")
    if not isinstance(params["volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `InputPathType`')
    if params.get("from-cropped", False) is None:
        raise StyxValidationError("`from-cropped` must not be None")
    if not isinstance(params["from-cropped"], bool):
        raise StyxValidationError(f'`from-cropped` has the wrong type: Received `{type(params.get("from-cropped", False))}` expected `bool`')


def cifti_replace_structure_volume_cargs(
    params: CiftiReplaceStructureVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-volume",
        params.get("structure", None),
        execution.input_file(params.get("volume", None))
    ])
    if params.get("from-cropped", False):
        cargs.append("-from-cropped")
    return cargs


class CiftiReplaceStructureOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiReplaceStructureParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def cifti_replace_structure_params(
    cifti: str,
    direction: str,
    volume_all: CiftiReplaceStructureVolumeAllParamsDict | None = None,
    label: list[CiftiReplaceStructureLabelParamsDict] | None = None,
    metric: list[CiftiReplaceStructureMetricParamsDict] | None = None,
    volume: list[CiftiReplaceStructureVolumeParamsDict] | None = None,
    action: str | None = None,
    discard_unused_labels: bool = False,
) -> CiftiReplaceStructureParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti: the cifti to modify.
        direction: which dimension to interpret as a single map, ROW or COLUMN.
        volume_all: replace the data in all volume components.
        label: replace the data in a surface label component.
        metric: replace the data in a surface component.
        volume: replace the data in a volume component.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'LEFT_SURFACE_FIRST', or 'LEGACY', default 'ERROR', use\
            'LEGACY' to match v1.4.2 and earlier.
        discard_unused_labels: when operating on a dlabel file, drop any unused\
            label keys from the label table.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-replace-structure",
        "cifti": cifti,
        "direction": direction,
        "discard-unused-labels": discard_unused_labels,
    }
    if volume_all is not None:
        params["volume-all"] = volume_all
    if label is not None:
        params["label"] = label
    if metric is not None:
        params["metric"] = metric
    if volume is not None:
        params["volume"] = volume
    if action is not None:
        params["action"] = action
    return params


def cifti_replace_structure_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiReplaceStructureParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti", None) is None:
        raise StyxValidationError("`cifti` must not be None")
    if not isinstance(params["cifti"], str):
        raise StyxValidationError(f'`cifti` has the wrong type: Received `{type(params.get("cifti", None))}` expected `str`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("volume-all", None) is not None:
        cifti_replace_structure_volume_all_validate(params["volume-all"])
    if params.get("label", None) is not None:
        if not isinstance(params["label"], list):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `list[CiftiReplaceStructureLabelParamsDict] | None`')
        for e in params["label"]:
            cifti_replace_structure_label_validate(e)
    if params.get("metric", None) is not None:
        if not isinstance(params["metric"], list):
            raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `list[CiftiReplaceStructureMetricParamsDict] | None`')
        for e in params["metric"]:
            cifti_replace_structure_metric_validate(e)
    if params.get("volume", None) is not None:
        if not isinstance(params["volume"], list):
            raise StyxValidationError(f'`volume` has the wrong type: Received `{type(params.get("volume", None))}` expected `list[CiftiReplaceStructureVolumeParamsDict] | None`')
        for e in params["volume"]:
            cifti_replace_structure_volume_validate(e)
    if params.get("action", None) is not None:
        if not isinstance(params["action"], str):
            raise StyxValidationError(f'`action` has the wrong type: Received `{type(params.get("action", None))}` expected `str | None`')
    if params.get("discard-unused-labels", False) is None:
        raise StyxValidationError("`discard-unused-labels` must not be None")
    if not isinstance(params["discard-unused-labels"], bool):
        raise StyxValidationError(f'`discard-unused-labels` has the wrong type: Received `{type(params.get("discard-unused-labels", False))}` expected `bool`')


def cifti_replace_structure_cargs(
    params: CiftiReplaceStructureParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-replace-structure"
    ])
    cargs.append(params.get("cifti", None))
    cargs.append(params.get("direction", None))
    if params.get("volume-all", None) is not None or params.get("label", None) is not None or params.get("metric", None) is not None or params.get("volume", None) is not None:
        cargs.extend([
            *(cifti_replace_structure_volume_all_cargs(params.get("volume-all", None), execution) if (params.get("volume-all", None) is not None) else []),
            *([a for c in [cifti_replace_structure_label_cargs(s, execution) for s in params.get("label", None)] for a in c] if (params.get("label", None) is not None) else []),
            *([a for c in [cifti_replace_structure_metric_cargs(s, execution) for s in params.get("metric", None)] for a in c] if (params.get("metric", None) is not None) else []),
            *([a for c in [cifti_replace_structure_volume_cargs(s, execution) for s in params.get("volume", None)] for a in c] if (params.get("volume", None) is not None) else [])
        ])
    if params.get("action", None) is not None:
        cargs.extend([
            "-label-collision",
            params.get("action", None)
        ])
    if params.get("discard-unused-labels", False):
        cargs.append("-discard-unused-labels")
    return cargs


def cifti_replace_structure_outputs(
    params: CiftiReplaceStructureParamsDict,
    execution: Execution,
) -> CiftiReplaceStructureOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiReplaceStructureOutputs(
        root=execution.output_file("."),
    )
    return ret


def cifti_replace_structure_execute(
    params: CiftiReplaceStructureParamsDict,
    runner: Runner | None = None,
) -> CiftiReplaceStructureOutputs:
    """
    REPLACE DATA IN A STRUCTURE IN A CIFTI FILE.
    
    This is a fairly low-level command, you probably want to use
    -cifti-create-dense-from-template instead.
    
    You must specify at least one of -metric, -label, -volume, or -volume-all
    for this command to do anything. Input volumes must line up with the output
    of -cifti-separate. For dtseries/dscalar, use COLUMN, and if your dconn
    matrix will be fully symmetric, COLUMN is more efficient. The -volume-all
    option must not be specified when using a -volume option. A -metric option
    must not be specified when using a -label option, and is not recommended on
    a label-type cifti file. For each <structure> argument, use one of the
    following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReplaceStructureOutputs`).
    """
    cifti_replace_structure_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_REPLACE_STRUCTURE_METADATA)
    params = execution.params(params)
    cargs = cifti_replace_structure_cargs(params, execution)
    ret = cifti_replace_structure_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_replace_structure(
    cifti: str,
    direction: str,
    volume_all: CiftiReplaceStructureVolumeAllParamsDict | None = None,
    label: list[CiftiReplaceStructureLabelParamsDict] | None = None,
    metric: list[CiftiReplaceStructureMetricParamsDict] | None = None,
    volume: list[CiftiReplaceStructureVolumeParamsDict] | None = None,
    action: str | None = None,
    discard_unused_labels: bool = False,
    runner: Runner | None = None,
) -> CiftiReplaceStructureOutputs:
    """
    REPLACE DATA IN A STRUCTURE IN A CIFTI FILE.
    
    This is a fairly low-level command, you probably want to use
    -cifti-create-dense-from-template instead.
    
    You must specify at least one of -metric, -label, -volume, or -volume-all
    for this command to do anything. Input volumes must line up with the output
    of -cifti-separate. For dtseries/dscalar, use COLUMN, and if your dconn
    matrix will be fully symmetric, COLUMN is more efficient. The -volume-all
    option must not be specified when using a -volume option. A -metric option
    must not be specified when using a -label option, and is not recommended on
    a label-type cifti file. For each <structure> argument, use one of the
    following strings:
    
    CORTEX_LEFT
    CORTEX_RIGHT
    CEREBELLUM
    ACCUMBENS_LEFT
    ACCUMBENS_RIGHT
    ALL_GREY_MATTER
    ALL_WHITE_MATTER
    AMYGDALA_LEFT
    AMYGDALA_RIGHT
    BRAIN_STEM
    CAUDATE_LEFT
    CAUDATE_RIGHT
    CEREBELLAR_WHITE_MATTER_LEFT
    CEREBELLAR_WHITE_MATTER_RIGHT
    CEREBELLUM_LEFT
    CEREBELLUM_RIGHT
    CEREBRAL_WHITE_MATTER_LEFT
    CEREBRAL_WHITE_MATTER_RIGHT
    CORTEX
    DIENCEPHALON_VENTRAL_LEFT
    DIENCEPHALON_VENTRAL_RIGHT
    HIPPOCAMPUS_LEFT
    HIPPOCAMPUS_RIGHT
    HIPPOCAMPUS_DENTATE_LEFT
    HIPPOCAMPUS_DENTATE_RIGHT
    INVALID
    OTHER
    OTHER_GREY_MATTER
    OTHER_WHITE_MATTER
    PALLIDUM_LEFT
    PALLIDUM_RIGHT
    PUTAMEN_LEFT
    PUTAMEN_RIGHT
    THALAMUS_LEFT
    THALAMUS_RIGHT.
    
    Args:
        cifti: the cifti to modify.
        direction: which dimension to interpret as a single map, ROW or COLUMN.
        volume_all: replace the data in all volume components.
        label: replace the data in a surface label component.
        metric: replace the data in a surface component.
        volume: replace the data in a volume component.
        action: how to handle conflicts between label keys\
            \
            'ERROR', 'LEFT_SURFACE_FIRST', or 'LEGACY', default 'ERROR', use\
            'LEGACY' to match v1.4.2 and earlier.
        discard_unused_labels: when operating on a dlabel file, drop any unused\
            label keys from the label table.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiReplaceStructureOutputs`).
    """
    params = cifti_replace_structure_params(
        cifti=cifti,
        direction=direction,
        volume_all=volume_all,
        label=label,
        metric=metric,
        volume=volume,
        action=action,
        discard_unused_labels=discard_unused_labels,
    )
    return cifti_replace_structure_execute(params, runner)


__all__ = [
    "CIFTI_REPLACE_STRUCTURE_METADATA",
    "CiftiReplaceStructureLabelParamsDict",
    "CiftiReplaceStructureLabelParamsDictTagged",
    "CiftiReplaceStructureMetricParamsDict",
    "CiftiReplaceStructureMetricParamsDictTagged",
    "CiftiReplaceStructureOutputs",
    "CiftiReplaceStructureParamsDict",
    "CiftiReplaceStructureParamsDictTagged",
    "CiftiReplaceStructureVolumeAllParamsDict",
    "CiftiReplaceStructureVolumeAllParamsDictTagged",
    "CiftiReplaceStructureVolumeParamsDict",
    "CiftiReplaceStructureVolumeParamsDictTagged",
    "cifti_replace_structure",
    "cifti_replace_structure_execute",
    "cifti_replace_structure_label",
    "cifti_replace_structure_metric",
    "cifti_replace_structure_params",
    "cifti_replace_structure_volume",
    "cifti_replace_structure_volume_all",
]
